// ----------------------------------
// Filename      : LinkIDBox.java
// Created       : 05/03/2000
// Last modified : 05/03/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.network.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class LinkIDBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private final int   kID = 0;
  private int         fID;
  private CfgInputBox fCfgID;
  private boolean     fCanceled;

  public LinkIDBox(Frame parent)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;

    // find the first used link-ID
    fID = 0;
    Link FindLink = (Link) fResources.fNetwork.fLinks.get(new Integer(fID));
    while (FindLink == null) {
      ++fID;
      FindLink = (Link) fResources.fNetwork.fLinks.get(new Integer(fID));
    }

    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textLinkIDBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    setLayout(new GridLayout(3,2));

    add(new Label(fMessages.lookup("textLinkID") + " = ",Label.RIGHT));
    fCfgID = new CfgInputBox(kID);
    add(fCfgID);

    add(new Label());
    add(new Label());

    Button ControlButton;
    ControlButton = new Button(fMessages.lookup("buttonCancel"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    ControlButton = new Button(fMessages.lookup("buttonOk"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  private void update()
  {
    fCfgID.update();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public int getLinkID()
  {
    return fID;
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInput;

    public CfgInputBox(int input)
    {
      super();
      fInput = input;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInput == kID) {
        setText(String.valueOf(fID));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int Value = 0;
      try {
        Value = (new Integer(e.getActionCommand())).intValue();
      }
      catch (NumberFormatException exc) {
        if (fInput == kID) {
          Value = fID;
        }
      }

      // check if the specified link-ID exists
      Link FindLink = (Link) fResources.fNetwork.fLinks.get(new Integer(Value));

      if (FindLink == null) {
        Value = fID;
      }

      if (fInput == kID) {
        fID = Value;
      }

      update();
    }
  }
}
