// ----------------------------------------
// Filename      : TranslateNetworkBox.java
// Created       : 11/02/200
// Last modified : 11/02/200
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class TranslateNetworkBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private int         fdx;
  private int         fdy;
  private int         fdz;
  private final int   kdx = 1;
  private final int   kdy = 2;
  private final int   kdz = 3;
  private CfgInputBox fCfgdx;
  private CfgInputBox fCfgdy;
  private CfgInputBox fCfgdz;
  private boolean     fCanceled;

  public TranslateNetworkBox(Frame parent)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fdx = 0;
    fdy = 0;
    fdz = 0;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textTranslateNetworkDialogTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel SettingsPanel = new Panel();
    Panel ControlPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(4,3));
    ControlPanel.setLayout(new GridLayout(1,2));

    SettingsPanel.add(new Label(fMessages.lookup("textTranslatedx") + " = ",Label.RIGHT));
    fCfgdx = new CfgInputBox(kdx);
    SettingsPanel.add(fCfgdx);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textTranslatedy") + " = ",Label.RIGHT));
    fCfgdy = new CfgInputBox(kdy);
    SettingsPanel.add(fCfgdy);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textTranslatedz") + " = ",Label.RIGHT));
    fCfgdz = new CfgInputBox(kdz);
    SettingsPanel.add(fCfgdz);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    // add the controlpanel
    setLayout(new BorderLayout());
    add(SettingsPanel,"North");
    add(ControlPanel,"South");

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public int getdx()
  {
    return fdx;
  }

  public int getdy()
  {
    return fdy;
  }

  public int getdz()
  {
    return fdz;
  }

  private void updateAllFields()
  {
    fCfgdx.update();
    fCfgdy.update();
    fCfgdz.update();
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kdx) {
        setText(String.valueOf(fdx));
      }
      else if (fInputBox == kdy) {
        setText(String.valueOf(fdy));
      }
      else if (fInputBox == kdz) {
        setText(String.valueOf(fdz));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int IntInput = 0;

      try {
        IntInput = (new Integer(e.getActionCommand())).intValue();
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kdx) {
          IntInput = fdx;
        }
        else if (fInputBox == kdy) {
          IntInput = fdy;
        }
        else if (fInputBox == kdz) {
          IntInput = fdz;
        }
      }

      if (fInputBox == kdx) {
        fdx = IntInput;
      }
      else if (fInputBox == kdy) {
        fdy = IntInput;
      }
      else if (fInputBox == kdz) {
        fdz = IntInput;
      }

      updateAllFields();
    }
  }
}
