// ----------------------------------
// Filename      : ConnectBox.java
// Created       : 07/03/2000
// Last modified : 22/04/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.util;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class ConnectBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private final int   kIPAddress = 0;
  private final int   kTCPPort   = 1;
  private String      fIPAddress;
  private int         fTCPPort;
  private CfgInputBox fCfgIPAddress;
  private CfgInputBox fCfgTCPPort;
  private boolean     fCanceled;

  public ConnectBox(Frame parent, String IPAddress, int TCPPort)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fIPAddress = IPAddress;
    fTCPPort = TCPPort;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textConnectBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    setLayout(new GridLayout(4,2));

    add(new Label(fMessages.lookup("textIPAddress") + " = ",Label.RIGHT));
    fCfgIPAddress = new CfgInputBox(kIPAddress);
    add(fCfgIPAddress);

    add(new Label(fMessages.lookup("textTCPPort") + " = ",Label.RIGHT));
    fCfgTCPPort = new CfgInputBox(kTCPPort);
    add(fCfgTCPPort);

    add(new Label());
    add(new Label());

    Button ControlButton;
    ControlButton = new Button(fMessages.lookup("buttonCancel"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    ControlButton = new Button(fMessages.lookup("buttonOk"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  private void update()
  {
    fCfgIPAddress.update();
    fCfgTCPPort.update();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public String getIPAddress()
  {
    return fIPAddress;
  }

  public int getTCPPort()
  {
    return fTCPPort;
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInput;

    public CfgInputBox(int input)
    {
      super();
      fInput = input;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInput == kIPAddress) {
        setText(fIPAddress);
      }
      else if (fInput == kTCPPort) {
        setText(String.valueOf(fTCPPort));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      String StringValue = "";
      int IntValue = 0;
      try {
        if (fInput == kIPAddress) {
          StringValue = e.getActionCommand();
        }
        else {
          IntValue = (new Integer(e.getActionCommand())).intValue();
        }
      }
      catch (NumberFormatException exc) {
        if (fInput == kTCPPort) {
          IntValue = fTCPPort;
        }
      }

      if (fInput == kIPAddress) {
        fIPAddress = StringValue;
      }
      else if (fInput == kTCPPort) {
        fTCPPort = IntValue;
      }

      update();
    }
  }
}
