// ----------------------------------
// Filename      : VLegend.java
// Created       : 22/06/2000
// Last modified : 22/06/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.visualizer;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class VLegend extends Frame
{
  // internal datastructures
  private Resources    fResources;
  private Messages     fMessages;
  private LegendCanvas fLegendCanvas;

  // only allow one instance of this class
  private static boolean fInstanceCreated;

  static {
    fInstanceCreated = false;
  }

  public VLegend()
  {
    super();

    if (fInstanceCreated) {
      return;
    }
    fInstanceCreated = true;

    fResources = new Resources();
    fMessages = fResources.fMessages;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setSize(fResources.fvisLegendXSize,fResources.fvisLegendYSize);
    setResizable(false);
    setLocation(100,100);
    setTitle(fMessages.lookup("textLegendWindowTitle"));
    setBackground(Color.lightGray);
    setForeground(Color.black);
    setIconImage(Toolkit.getDefaultToolkit().getImage("M2000Icon.jpg"));

    setLayout(new BorderLayout());
    fLegendCanvas = new LegendCanvas(fResources.fvisLegendXSize - 20,
                                     fResources.fvisLegendYSize - 20);
    add("Center",fLegendCanvas);

    pack();
    show();
  }

  public void repaintLegend()
  {
    fLegendCanvas.repaint();
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  // the message-canvas
  private class LegendCanvas extends Canvas
  {
    // the canvas-draw-area
    private int cx1;
    private int cy1;
    private int cx2;
    private int cy2;
    private int cwidth;
    private int cheight;

    // internal datastructure
    private ColorInterpolator fColorInterpolator;

    public LegendCanvas(int width, int height)
    {
      super();

      setSize(width,height);
      fColorInterpolator = new ColorInterpolator();
    }

    public void update(Graphics g)
    {
      paint(g);
    }

    public void paint(Graphics g)
    {
      // define the boundaries for drawing
      cx1 = 5;
      cy1 = 5;
      cx2 = (cx1 + getSize().width) - (getInsets().right + 5 + 5);
      cy2 = (cy1 + getSize().height) - (getInsets().bottom + 5 + 5);
      cwidth = cx2 - cx1;
      cheight = cy2 - cy1;

      // activate clipping of graphics primitives
      g.setClip(cx1,cy1,cwidth,cheight);

      // clear the canvas
      g.setColor(Color.lightGray);
      g.fillRect(cx1,cy1,cwidth,cheight);

      if (fResources.fvisShowTrafficFlow) {

        if (fResources.fvisAnimateVehicles) {

          if (fResources.fvisColorVehiclesByAggression) {

            showColorBar(g,fColorInterpolator.interpolate(0),
                         cy1 + 10,fMessages.lookup("textAggressive"));
            showColorBar(g,fColorInterpolator.interpolate(10),
                         cy1 + 30,"");
            showColorBar(g,fColorInterpolator.interpolate(20),
                         cy1 + 50,"");
            showColorBar(g,fColorInterpolator.interpolate(30),
                         cy1 + 70,"");
            showColorBar(g,fColorInterpolator.interpolate(40),
                         cy1 + 90,"");
            showColorBar(g,fColorInterpolator.interpolate(50),
                         cy1 + 110,"");
            showColorBar(g,fColorInterpolator.interpolate(60),
                         cy1 + 130,"");
            showColorBar(g,fColorInterpolator.interpolate(70),
                         cy1 + 150,"");
            showColorBar(g,fColorInterpolator.interpolate(80),
                         cy1 + 170,"");
            showColorBar(g,fColorInterpolator.interpolate(90),
                         cy1 + 190,"");
            showColorBar(g,fColorInterpolator.interpolate(100),
                         cy1 + 210,fMessages.lookup("textConservative"));

            g.setColor(Color.black);
            g.drawLine(cx1 + 110,cy1 + 30,cx1 + 110,cy1 + 205);

            g.drawLine(cx1 + 110,cy1 + 30,cx1 + 100,cy1 + 40);
            g.drawLine(cx1 + 110,cy1 + 30,cx1 + 120,cy1 + 40);

            g.drawLine(cx1 + 110,cy1 + 205,cx1 + 100,cy1 + 195);
            g.drawLine(cx1 + 110,cy1 + 205,cx1 + 120,cy1 + 195);
          }
          else {

            g.setColor(fResources.kvisCarColor);
            g.fillRect(cx1 + 10,cy1 + 10,14,15);
            g.setColor(fResources.kvisCarEdgeColor);
            g.drawRect(cx1 + 10,cy1 + 10,14,15);

            g.setColor(Color.black);
            g.drawString(fMessages.lookup("textCar"),cx1 + 80,cy1 + 10 + 12);

            g.setColor(fResources.kvisTruckColor);
            g.fillRect(cx1 + 10,cy1 + 30,60,15);
            g.setColor(fResources.kvisTruckEdgeColor);
            g.drawRect(cx1 + 10,cy1 + 30,60,15);

            g.setColor(Color.black);
            g.drawString(fMessages.lookup("textTruck"),cx1 + 80,cy1 + 30 + 12);

            g.setColor(fResources.kvisRoadSurfaceColor);
            g.fillRect(cx1 + 10,cy1 + 50,60,15);
            g.setColor(fResources.kvisRoadExtentColor);
            g.drawRect(cx1 + 10,cy1 + 50,60,15);

            g.setColor(Color.black);
            g.drawString(fMessages.lookup("textRoad"),cx1 + 80,cy1 + 50 + 12);
          }
        }
        else if (fResources.fvisShowAverageSpeeds) {

          showColorBar(g,fColorInterpolator.interpolate(0),
                       cy1 + 10,fMessages.lookup("textLessThan10Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(10),
                       cy1 + 30,fMessages.lookup("text10To19Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(20),
                       cy1 + 50,fMessages.lookup("text20To29Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(30),
                       cy1 + 70,fMessages.lookup("text30To39Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(40),
                       cy1 + 90,fMessages.lookup("text40To49Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(50),
                       cy1 + 110,fMessages.lookup("text50To59Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(60),
                       cy1 + 130,fMessages.lookup("text60To69Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(70),
                       cy1 + 150,fMessages.lookup("text70To79Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(80),
                       cy1 + 170,fMessages.lookup("text80To89Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(90),
                       cy1 + 190,fMessages.lookup("text90To99Kmh"));
          showColorBar(g,fColorInterpolator.interpolate(100),
                       cy1 + 210,fMessages.lookup("text100AndUpKmh"));
        }
        else if (fResources.fvisShowDensities) {

          showColorBar(g,fColorInterpolator.interpolate(0),
                       cy1 + 10,fMessages.lookup("text100AndUpVehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(10),
                       cy1 + 30,fMessages.lookup("text90To99VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(20),
                       cy1 + 50,fMessages.lookup("text80To89VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(30),
                       cy1 + 70,fMessages.lookup("text70To79VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(40),
                       cy1 + 90,fMessages.lookup("text60To69VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(50),
                       cy1 + 110,fMessages.lookup("text50To59VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(60),
                       cy1 + 130,fMessages.lookup("text40To49VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(70),
                       cy1 + 150,fMessages.lookup("text30To39VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(80),
                       cy1 + 170,fMessages.lookup("text20To29VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(90),
                       cy1 + 190,fMessages.lookup("text10To19VehKmLane"));
          showColorBar(g,fColorInterpolator.interpolate(100),
                       cy1 + 210,fMessages.lookup("textLessThan10VehKmLane"));
        }
      }
      else { // showing statistics
          g.setColor(Color.black);
          g.drawString(fMessages.lookup("messageNoLegendAvailable"),cx1 + 10,cy1 + 10 + 12);
      }
    }

    private void showColorBar(Graphics g, Color color, int y, String Description)
    {
      g.setColor(color);
      g.fillRect(cx1 + 10,y,60,15);
      g.setColor(Color.black);
      g.drawString(Description,cx1 + 80,y + 12);
    }
  }
}
