// -----------------------------------------
// Filename      : ViewSpecificationBox.java
// Created       : 24/04/2000
// Last modified : 20/05/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// -----------------------------------------

package mitrasim.visualizer;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import mitrasim.engine3d.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class ViewSpecificationBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;

  private final int   kViewSpecification   = 1;
  private final int   kProjectionMode      = 2;
  private final int   kViewPointX          = 3;
  private final int   kViewPointY          = 4;
  private final int   kViewPointZ          = 5;
  private final int   kEyePointRho         = 6;
  private final int   kEyePointPhi         = 7;
  private final int   kEyePointTheta       = 8;
  private final int   kBoundingBoxMinX     = 9;
  private final int   kBoundingBoxMinY     = 10;
  private final int   kBoundingBoxMaxX     = 11;
  private final int   kBoundingBoxMaxY     = 12;
  private final int   kShowEnvironment     = 13;

  private boolean     fAllFieldsDefined;
  private CfgItemList fCfgViewSpecification;
  private CfgItemList fCfgProjectionMode;
  private CfgInputBox fCfgViewPointX;
  private CfgInputBox fCfgViewPointY;
  private CfgInputBox fCfgViewPointZ;
  private CfgInputBox fCfgEyePointRho;
  private CfgInputBox fCfgEyePointPhi;
  private CfgInputBox fCfgEyePointTheta;
  private CfgInputBox fCfgBoundingBoxMinX;
  private CfgInputBox fCfgBoundingBoxMinY;
  private CfgInputBox fCfgBoundingBoxMaxX;
  private CfgInputBox fCfgBoundingBoxMaxY;
  private CfgCheckBox fCfgShowEnvironment;

  private String      fOldViewSpecification;
  private boolean     fOldPerspectiveMode;
  private int         fOldViewPointX;
  private int         fOldViewPointY;
  private int         fOldViewPointZ;
  private int         fOldEyePointRho;
  private int         fOldEyePointPhi;
  private int         fOldEyePointTheta;
  private int         fOldBoundingBoxMinX;
  private int         fOldBoundingBoxMinY;
  private int         fOldBoundingBoxMaxX;
  private int         fOldBoundingBoxMaxY;
  private boolean     fOldShowEnvironment;

  private boolean     fCanceled;

  public ViewSpecificationBox(Frame parent)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fOldViewSpecification = fResources.fvisViewSpecification;
    fOldPerspectiveMode   = fResources.fvisPerspectiveMode;
    fOldViewPointX        = fResources.fvisViewPointX;
    fOldViewPointY        = fResources.fvisViewPointY;
    fOldViewPointZ        = fResources.fvisViewPointZ;
    fOldEyePointRho       = fResources.fvisEyePointRho;
    fOldEyePointPhi       = fResources.fvisEyePointPhi;
    fOldEyePointTheta     = fResources.fvisEyePointTheta;
    fOldBoundingBoxMinX   = fResources.fvisBoundingBoxMinX;
    fOldBoundingBoxMinY   = fResources.fvisBoundingBoxMinY;
    fOldBoundingBoxMaxX   = fResources.fvisBoundingBoxMaxX;
    fOldBoundingBoxMaxY   = fResources.fvisBoundingBoxMaxY;
    fOldShowEnvironment   = fResources.fvisShowEnvironment;

    fAllFieldsDefined = false;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textViewSpecificationBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // create the configuration-panel
    Panel CfgPanel = new Panel();
    CfgPanel.setLayout(new GridLayout(18,3));

    Label ViewSpecificationLabel = new Label(fMessages.lookup("textViewSpecification") + " : ",Label.RIGHT);
    CfgPanel.add(ViewSpecificationLabel);
    fCfgViewSpecification = new CfgItemList(kViewSpecification);
    CfgPanel.add(fCfgViewSpecification);
    CfgPanel.add(new Label());

    if (fResources.fvisViewSpecifications.size() == 0) {
      ViewSpecificationLabel.setEnabled(false);
      fCfgViewSpecification.setEnabled(false);
    }

    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));

    CfgPanel.add(new Label(fMessages.lookup("textProjectionMode") + " : ",Label.RIGHT));
    fCfgProjectionMode = new CfgItemList(kProjectionMode);
    CfgPanel.add(fCfgProjectionMode);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));

    CfgPanel.add(new Label(fMessages.lookup("textViewPointX") + " : ",Label.RIGHT));
    fCfgViewPointX = new CfgInputBox(kViewPointX);
    CfgPanel.add(fCfgViewPointX);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textViewPointY") + " : ",Label.RIGHT));
    fCfgViewPointY = new CfgInputBox(kViewPointY);
    CfgPanel.add(fCfgViewPointY);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textViewPointZ") + " : ",Label.RIGHT));
    fCfgViewPointZ = new CfgInputBox(kViewPointZ);
    CfgPanel.add(fCfgViewPointZ);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));

    CfgPanel.add(new Label(fMessages.lookup("textEyePointRho") + " : ",Label.RIGHT));
    fCfgEyePointRho = new CfgInputBox(kEyePointRho);
    CfgPanel.add(fCfgEyePointRho);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textEyePointPhi") + " : ",Label.RIGHT));
    fCfgEyePointPhi = new CfgInputBox(kEyePointPhi);
    CfgPanel.add(fCfgEyePointPhi);
    CfgPanel.add(new Label(" " + fMessages.lookup("textRotateUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textEyePointTheta") + " : ",Label.RIGHT));
    fCfgEyePointTheta = new CfgInputBox(kEyePointTheta);
    CfgPanel.add(fCfgEyePointTheta);
    CfgPanel.add(new Label(" " + fMessages.lookup("textRotateUnit"),Label.LEFT));

    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));

    CfgPanel.add(new Label(fMessages.lookup("textBoundingBoxMinX") + " : ",Label.RIGHT));
    fCfgBoundingBoxMinX = new CfgInputBox(kBoundingBoxMinX);
    CfgPanel.add(fCfgBoundingBoxMinX);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textBoundingBoxMinY") + " : ",Label.RIGHT));
    fCfgBoundingBoxMinY = new CfgInputBox(kBoundingBoxMinY);
    CfgPanel.add(fCfgBoundingBoxMinY);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textBoundingBoxMaxX") + " : ",Label.RIGHT));
    fCfgBoundingBoxMaxX = new CfgInputBox(kBoundingBoxMaxX);
    CfgPanel.add(fCfgBoundingBoxMaxX);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textBoundingBoxMaxY") + " : ",Label.RIGHT));
    fCfgBoundingBoxMaxY = new CfgInputBox(kBoundingBoxMaxY);
    CfgPanel.add(fCfgBoundingBoxMaxY);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));
    CfgPanel.add(new Label(""));

    CfgPanel.add(new Label(fMessages.lookup("textShowEnvironment") + " : ",Label.RIGHT));
    fCfgShowEnvironment = new CfgCheckBox(kShowEnvironment);
    CfgPanel.add(fCfgShowEnvironment);
    CfgPanel.add(new Label());

    fAllFieldsDefined = true;

    // create the control-panel
    Panel ControlPanel = new Panel();
    ControlPanel.setLayout(new GridLayout(1,2));

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    setLayout(new BorderLayout());
    add("North",CfgPanel);
    add("South",ControlPanel);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();

    updateAllFields();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  private void updateAllFields()
  {
    // don't update the fCfgViewSpecification in order to prevent
    // overwriting of the other values
    fCfgProjectionMode.update();
    fCfgViewPointX.update();
    fCfgViewPointY.update();
    fCfgViewPointZ.update();
    fCfgEyePointRho.update();
    fCfgEyePointPhi.update();
    fCfgEyePointTheta.update();
    fCfgBoundingBoxMinX.update();
    fCfgBoundingBoxMinY.update();
    fCfgBoundingBoxMaxX.update();
    fCfgBoundingBoxMaxY.update();
    fCfgShowEnvironment.update();
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    if (fCanceled) {
      // restore all the old values
      fResources.fvisPerspectiveMode = fOldPerspectiveMode;
      fResources.fvisViewPointX      = fOldViewPointX;
      fResources.fvisViewPointY      = fOldViewPointY;
      fResources.fvisViewPointZ      = fOldViewPointZ;
      fResources.fvisEyePointRho     = fOldEyePointRho;
      fResources.fvisEyePointPhi     = fOldEyePointPhi;
      fResources.fvisEyePointTheta   = fOldEyePointTheta;
      fResources.fvisBoundingBoxMinX = fOldBoundingBoxMinX;
      fResources.fvisBoundingBoxMinY = fOldBoundingBoxMinX;
      fResources.fvisBoundingBoxMaxX = fOldBoundingBoxMaxX;
      fResources.fvisBoundingBoxMaxY = fOldBoundingBoxMaxY;
      fResources.fvisShowEnvironment = fOldShowEnvironment;
    }
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgCheckBox extends Checkbox implements ItemListener
  {
    private int fCheckBox;

    public CfgCheckBox(int checkBox)
    {
      super();
      fCheckBox = checkBox;
      addItemListener(this);
      update();
    }

    synchronized public void update()
    {
      if (fCheckBox == kShowEnvironment) {
        setState(fResources.fvisShowEnvironment);
        if (fResources.fvisShowEnvironment) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fCheckBox == kShowEnvironment) {
        fResources.fvisShowEnvironment = !fResources.fvisShowEnvironment;
      }

      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kViewPointX) {
        setText(String.valueOf(fResources.fvisViewPointX));
      }
      else if (fInputBox == kViewPointY) {
        setText(String.valueOf(fResources.fvisViewPointY));
      }
      else if (fInputBox == kViewPointZ) {
        setText(String.valueOf(fResources.fvisViewPointZ));
      }
      else if (fInputBox == kEyePointRho) {
        setText(String.valueOf(fResources.fvisEyePointRho));
      }
      else if (fInputBox == kEyePointPhi) {
        setText(String.valueOf(fResources.fvisEyePointPhi));
      }
      else if (fInputBox == kEyePointTheta) {
        setText(String.valueOf(fResources.fvisEyePointTheta));
      }
      else if (fInputBox == kBoundingBoxMinX) {
        setText(String.valueOf(fResources.fvisBoundingBoxMinX));
      }
      else if (fInputBox == kBoundingBoxMinY) {
        setText(String.valueOf(fResources.fvisBoundingBoxMinY));
      }
      else if (fInputBox == kBoundingBoxMaxX) {
        setText(String.valueOf(fResources.fvisBoundingBoxMaxX));
      }
      else if (fInputBox == kBoundingBoxMaxY) {
        setText(String.valueOf(fResources.fvisBoundingBoxMaxY));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int NewInput = 0;
      try {
        NewInput = (new Integer(e.getActionCommand())).intValue();
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kViewPointX) {
          NewInput = fResources.fvisViewPointX;
        }
        else if (fInputBox == kViewPointY) {
          NewInput = fResources.fvisViewPointY;
        }
        else if (fInputBox == kViewPointZ) {
          NewInput = fResources.fvisViewPointZ;
        }
        else if (fInputBox == kEyePointRho) {
          NewInput = fResources.fvisEyePointRho;
        }
        else if (fInputBox == kEyePointPhi) {
          NewInput = fResources.fvisEyePointPhi;
        }
        else if (fInputBox == kEyePointTheta) {
          NewInput = fResources.fvisEyePointTheta;
        }
        else if (fInputBox == kBoundingBoxMinX) {
          NewInput = fResources.fvisBoundingBoxMinX;
        }
        else if (fInputBox == kBoundingBoxMinY) {
          NewInput = fResources.fvisBoundingBoxMinY;
        }
        else if (fInputBox == kBoundingBoxMaxX) {
          NewInput = fResources.fvisBoundingBoxMaxX;
        }
        else if (fInputBox == kBoundingBoxMaxY) {
          NewInput = fResources.fvisBoundingBoxMaxY;
        }
      }

      if (fInputBox == kViewPointX) {
        fResources.fvisViewPointX = NewInput;
      }
      else if (fInputBox == kViewPointY) {
        fResources.fvisViewPointY = NewInput;
      }
      else if (fInputBox == kViewPointZ) {
        fResources.fvisViewPointZ = NewInput;
      }
      else if (fInputBox == kEyePointRho) {
        if (NewInput < 0) {
          NewInput = -NewInput;
        }
        fResources.fvisEyePointRho = NewInput;
      }
      else if (fInputBox == kEyePointPhi) {
        if (NewInput > 180) {
          NewInput = 180;
        }
        else if (NewInput < -180) {
          NewInput = -180;
        }
        fResources.fvisEyePointPhi = NewInput;
      }
      else if (fInputBox == kEyePointTheta) {
        if (NewInput > 359) {
          NewInput = 359;
        }
        else if (NewInput < 0) {
          NewInput = 0;
        }
        fResources.fvisEyePointTheta = NewInput;
      }
      else if (fInputBox == kBoundingBoxMinX) {
        fResources.fvisBoundingBoxMinX = NewInput;
      }
      else if (fInputBox == kBoundingBoxMinY) {
        fResources.fvisBoundingBoxMinY = NewInput;
      }
      else if (fInputBox == kBoundingBoxMaxX) {
        fResources.fvisBoundingBoxMaxX = NewInput;
      }
      else if (fInputBox == kBoundingBoxMaxY) {
        fResources.fvisBoundingBoxMaxY = NewInput;
      }

      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgItemList extends Choice implements ItemListener
  {
    private int fItemList;

    public CfgItemList(int itemList)
    {
      super();
      fItemList = itemList;
      if (fItemList == kViewSpecification) {
        if (fResources.fvisViewSpecifications.size() == 0) {
          return;
        }
        Enumeration ViewSpecificationEnum = fResources.fvisViewSpecifications.elements();
        boolean FirstItem = true;
        while (ViewSpecificationEnum.hasMoreElements()) {
          ViewSpecification viewSpecification = (ViewSpecification) ViewSpecificationEnum.nextElement();
          add(viewSpecification.fName);
          if (FirstItem) {
            fResources.fvisViewSpecification = viewSpecification.fName;
            FirstItem = false;
          }
        }
      }
      else if (fItemList == kProjectionMode) {
        add(fResources.fMessages.lookup("textPerspectiveProjection"));
        add(fResources.fMessages.lookup("textParallelProjection"));
      }
      update();
      addItemListener(this);
    }

    synchronized public void update()
    {
      if (fItemList == kViewSpecification) {
        if (fResources.fvisViewSpecifications.size() == 0) {
          return;
        }
        select(fResources.fvisViewSpecification);
      }
      else if (fItemList == kProjectionMode) {
        if (fResources.fvisPerspectiveMode) {
          select(fResources.fMessages.lookup("textPerspectiveProjection"));
        }
        else {
          select(fResources.fMessages.lookup("textParallelProjection"));
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fItemList == kViewSpecification) {
        if (fResources.fvisViewSpecifications.size() == 0) {
          return;
        }
        fResources.fvisViewSpecification = (String) e.getItem();

        ViewSpecification viewSpecification = (ViewSpecification) fResources.fvisViewSpecifications.get(new String(fResources.fvisViewSpecification));

        if (viewSpecification != null) {
          fResources.fvisPerspectiveMode = viewSpecification.fPerspectiveMode;
          fResources.fvisViewPointX = viewSpecification.fViewPointX;
          fResources.fvisViewPointY = viewSpecification.fViewPointY;
          fResources.fvisViewPointZ = viewSpecification.fViewPointZ;
          fResources.fvisEyePointRho = viewSpecification.fEyePointRho;
          fResources.fvisEyePointPhi = viewSpecification.fEyePointPhi;
          fResources.fvisEyePointTheta = viewSpecification.fEyePointTheta;
          fResources.fvisBoundingBoxMinX = viewSpecification.fBoundingBoxMinX;
          fResources.fvisBoundingBoxMinY = viewSpecification.fBoundingBoxMinY;
          fResources.fvisBoundingBoxMaxX = viewSpecification.fBoundingBoxMaxX;
          fResources.fvisBoundingBoxMaxY = viewSpecification.fBoundingBoxMaxY;
          fResources.fvisShowEnvironment = viewSpecification.fShowEnvironment;

          // update the other fields
          if (fAllFieldsDefined) {
            fCfgProjectionMode.update();
            fCfgViewPointX.update();
            fCfgViewPointY.update();
            fCfgViewPointZ.update();
            fCfgEyePointRho.update();
            fCfgEyePointPhi.update();
            fCfgEyePointTheta.update();
            fCfgBoundingBoxMinX.update();
            fCfgBoundingBoxMinY.update();
            fCfgBoundingBoxMaxX.update();
            fCfgBoundingBoxMaxY.update();
          }
        }
      }
      else if (fItemList == kProjectionMode) {
        String ProjectionMode = (String) e.getItem();
        if (ProjectionMode.equalsIgnoreCase(fResources.fMessages.lookup("textPerspectiveProjection"))) {
          fResources.fvisPerspectiveMode = true;
        }
        else {
          fResources.fvisPerspectiveMode = false;
        }
      }
      updateAllFields();
    }

    public Dimension getPreferredSize()
    {
      return new Dimension(250,25);
    }

    public Dimension getMinimumSize()
    {
      return getPreferredSize();
    }
  }
}
